#!/usr/local/student/bin/scty -f
##############
# Ethan Gold, 3/12/96 - etgold@cs.vassar.edu
# this script uses a configuration file
# that is designated at startup by "netmonitor configfile" OR that
# resides in the user's home directory netmonitor.conf
#    the conf file is composed of lines of space seperated records
# hostname round_trip_time warning_level_string notification_type notification_parameters comments
# the logfile is also designated in the config file
#    most parameters should be configurable from within the config
# file or within the immediatly following variable block. the field
# numbers are set here so as to simplify the addition of new field
# into the config file.
#############

set mailer "/usr/ucb/mail"
set configfilename "netmonitor.conf"
set configfilepath /usr/local/etc/
#set configfilepath $env(HOME)
#set logfile "netmonitor.log"

set tknotify "/usr/local/bin/tknotify"
set delay 20
set threshhold 3
set hostfield 0
set timeoutfield 1
set warnlevelfield 2
set notifyfield 3
set notifyparamfield 4
set last_config_field 4


#######################################
# check for command line config file
# otherwise open predefined config file
# for reading.
#######################################

if { $argc == 1 } {
    if { [file exists [lindex $argv 0]] } {
	set configfile [open [lindex $argv 0] r]
    } else {
	puts "[lindex $argv 0] not found. opening default config file."
	set configfile [open $configfilepath/$configfilename r]
    }
} else {
    set configfile [open $configfilepath/$configfilename r]
}

#set logfile [open $configfilepath/$logfilename r+ ]

#######################################
# read in the config
# file from disk and store in an array
# indexed by host name
#######################################
while { [gets $configfile line] >= 0 } {
    if { [regexp {^#.*} $line] == 0 && [regexp {^$} $line] == 0 } {
	if { [regexp {^logfile\ .*$} $line] } {
	    if {[file exists [lindex $line 1]] == 0 } {
		exec touch [lindex $line 1]
	    }
	    set logfile [lindex $line 1]
	    puts "netmonitor logfile: $logfile"
	} elseif { [regexp {^delay\ .*$} $line] } {
	    set delay [lindex $line 1]
	} elseif { [regexp {^threshhold\ .*$} $line] } {
	    set threshhold [lindex $line 1]
	} else {
	    set host_array([lindex $line 0 ]) [ lrange $line 1 end ]
	    set error_array([lindex $line 0]) 0
	}
    }
}
close $configfile


## set field values to what the will be in the host_array
## i.e. $value - 1
set timeoutfield [expr $timeoutfield - 1]
set warnlevelfield [expr $warnlevelfield - 1]
set notifyfield [expr $notifyfield - 1]
set notifyparamfield [expr $notifyparamfield - 1]
set last_config_field [expr $last_config_field - 1]

#### log startup to the logfile
exec echo "\nnetmonitor STARTING UP at [getdate] with PID [pid]\n" >> $logfile

#### reset problem history counter(s)
set problem 0
set cycle 3

#######################################
# check the reachability of the hosts in
# the array and log error's to the logfile
# increment the problem counter for each
# host that cannot be reached
#######################################
proc checkreach { } {
    global host_array error_array logfile problem
    global threshhold timeoutfield warnlevelfield
    #puts "in checkreach"
    foreach host [ array names host_array ] {
	set far_away [ lindex $host_array($host) $timeoutfield ]
	set warning_level [ lindex $host_array($host) $warnlevelfield ]
	set time [ lindex [ lindex [icmp -timeout 2 -delay 1 echo $host] 0] 1 ]
	if { $time == -1 } {
	    exec echo "$warning_level: $host is unreachable at [getdate]" \
		    >> $logfile
	    incr error_array($host)
	    #puts "$warning_level: $host is unreachable at [getdate]"
	} elseif { $error_array($host) > 0 } {
	    exec echo "$host recovered at [getdate]" >> $logfile
	    if { $error_array($host) >= $threshhold } {
		set error_array($host) -1
	    } else {
		set error_array($host) 0
	    }
	} elseif { $time > $far_away } {
	    exec echo "$host is $time time units away at [getdate]" >> $logfile
	    set error_array($host) 0
	} else {
	    set error_array($host) 0
	}
    }
}


#######################################
# the notification procedures
#######################################

proc notification1 { host bad } {
    ### pop's up a window with the log in it and talks out the speaker
    if { $bad == 1 } {
	global logfile
	exec $tknotify -display :0.0 -geometry +200+200 -name "Netmonitor Alert: $host" -f $logfile &
	exec say "yo. something is rong." &
    } else {
	exec say "never mind" &
    }
}

proc notification2 { host bad } {
    ### mails error's to specified address and mails
    ### recovery messages as well
    global host_array notifyparamfield warnlevelfield mailer
    set warning_level [lindex $host_array($host) $warnlevelfield]
    regsub -all " " [getdate] "" dateforname
    set mailtmp [ open /tmp/$dateforname.netmonitor w ]
    
    if { $bad == 1 } {
	puts $mailtmp "$warning_level: $host is unreachable at [getdate]"
	close $mailtmp
	#exec echo "$warning_level: $host is unreachable at [getdate]" \
	#	>> /tmp/$dateforname.netmonitor
	foreach address [ lrange $host_array($host) $notifyparamfield end ] {
	    exec $mailer -s "network_error" $address < /tmp/$dateforname.netmonitor
	}
    } else {
	puts $mailtmp "$host recovered at [getdate]"
	close $mailtmp
	#exec echo "$host recovered at [getdate]" \
		#	>> /tmp/$dateforname.netmonitor
	foreach address [ lrange $host_array($host) $notifyparamfield end ] {
	    exec $mailer -s "network_recovery" $address < /tmp/$dateforname.netmonitor
	}
	
    }
    
    exec /bin/rm -f  /tmp/$dateforname.netmonitor
    
}
 
proc notification3 { host bad } {
    global host_array notifyparamfield warnlevelfield
    foreach field [lrange $host_array($host) $notifyparamfield end] {
	if { $bad == 1 } {
	    notification1 host $bad
	} else {
	    notification1 host $bad
	}
    }
}


#######################################
# a problem has happened. do the appropriate
# notification
#######################################
proc notify { host bad } {
    #puts "in notify"
    global logfile host_array notifyfield warnlevelfield
    global notifyparamfield
    #puts "$host [lindex $host_array($host) $notifyfield ]"
    switch -exact -- [ lindex $host_array($host) $notifyfield ] {
	notification1 { notification1 $host $bad }
	notification2 { notification2 $host $bad }
	notification3 { notification3 $host $bad } 
	default { notification1 $host $bad }
    }

}


#######################################
# the main event loop. check the hosts
# every $delay seconds and send notification
# if $problem is greater than $threshhold
#######################################
while {1} {
    #puts "main loop"
    checkreach 
    exec sleep $delay
    foreach host [ array names host_array ] {
	if { $error_array($host) == $threshhold } {
	    notify $host 1
	    exec echo "[lindex $host_array($host) $notifyfield] sent for $host at [getdate]" >> $logfile
	}
	if { $error_array($host) == -1 } {
	    notify $host 0
	        exec echo "recovery [lindex $host_array($host) $notifyfield] sent for $host at [getdate]" >> $logfile
	}
    }
    #puts "end of main loop"
}

exit


